﻿//DMLHM Dynamic Multi-Level Horizontal Menu version 0.1.5
//
//Copyright 2010 Alexis Salzman
//
//This program is free software: you can redistribute it and/or modify
//it under the terms of the GNU General Public License as published by
//the Free Software Foundation, either version 3 of the License, or
//(at your option) any later version.
//
//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with this program.  If not, see <http://www.gnu.org/licenses/>.
	
/**
 * Use of anonymous function to alias jQuery to $ and isolate this object from the global context. This function is executed immediately with "jQuery" passed to it. You can use whatever alias for jQuery you like. 
 * @name anonymous
 * @function
 * @param {object} $ The name of this parameter aliased jQuery object. Here it is aliased to $.
 */
(function($) {

	/**
	 * jQuery object's methods
	 * @name fn
	 * @memberOf $
	 * @see anonymous
	 * @namespace jQuery user methods container (plungin)
	 */
	/**
	 * <p>DMLHM Dynamic Multi-Level Horizontal Menu</p>
	 * <p> Mainly this method transform a standard menu list in a tree menu made of DIV. Only part of the tree is visible. It correspond to a branch of the tree made of the sub menu and its parents.</p> 
	 * <p>This method is attached to the jQuery.fn object like all plugin. In fact it is the aliased parameter $ of the anonymous function witch is used to reach fn</p>
	 * <p>Copyright 2010 Alexis Salzman</p>
	 * <p class="lic">This program is free software: you can redistribute it and/or modify
	 * it under the terms of the GNU General Public License as published by
	 * the Free Software Foundation, either version 3 of the License, or
	 * (at your option) any later version.</p>
	 * 
	 * <p class="lic">This program is distributed in the hope that it will be useful,
	 * but WITHOUT ANY WARRANTY; without even the implied warranty of
	 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	 * GNU General Public License for more details.</p>
	 * 
	 * <p class="lic">You should have received a copy of the GNU General Public License
	 * along with this program.  If not, see <a href="http://www.gnu.org/licenses/">http://www.gnu.org/licenses/</a>.</p>
	 * @function
	 * @name dmlhm
	 * @version 0.1.5
	 * @memberOf $.fn
	 * @param  {obj} options Parameter object witch set different value for defaults options.
	 * @returns {obj[]} Return the collection of jQuery object transformed by dmlhm method.
	 * @namespace  Pluging DMLHM
	 */
	$.fn.dmlhm = function(options) {

		var o = $.extend({}, $.fn.dmlhm.defaults, options);

		/**
		 * calculate the sum of increasing integer up to the parameter value
		 * @name $.fn.dmlhm#sum_n
		 * @function
		 * @since version 0.1.0 
		 * @inner
		 * @param  {int} n End value for the sum.
		 * @returns {int} Sum of 1,2,....n
		 */
		function sum_n(n) {
			var sum_=0;
			for(var i=0;i<=n;i++) {
				sum_ += i;
			}
			return sum_;      	
		};
		/**
		 * Calculate coefficients for the size_item function. This last function use a 
		 * linear equation to obtain the size of a item menu depending on the context. 
		 * Here, depending on the number of left (m) an right (n) items 
		 * around the selected item, this function calculate 'a' and 'b' (y=ax+b) and use the following logic :
		 * <ol><li>
		 * It use x0 (the min_width given by the options) for items directly around the selected
		 * one. The others will have a decreasing size. The slope of the linear equation is
		 * calculated so that the (sub)menu fill the div container (=>the largest items possible).
		 * </li><li>
		 * If there is too many menu items or the size of the div container (S) is to small, starting from selected item 
		 * with one or two items of size x0 don't lead to correct size for over items. There size becomes less
		 * then min_width_abs+border (or less then 0 !). x0 in this case have to be recalculated. It is obtained by
		 * considering that the furthest item (far mean here far from the selected item) have a minimum size of min_width_abs+border (coef.d).
		 * </ol>
		 * <p>Note: the case where all unselected item may have a size of x0 is not treated here but in the calling function (function attached to resize event)</p> 
		 * @name $.fn.dmlhm#calc_coef
		 * @function
		 * @since version 0.1.0 
		 * @inner
		 * @param {int} n number of items at the right of the selected item
		 * @param {int} m number of items at the left of the selected item
		 * @param {int} S size of the menu item container
		 * @param {int} x0 initial size for the first items at the left or right of the selected item
		 * @param {int} Mx size of the selected item
		 * @param {obj} coef coefficient used by size_item to calculate the size of a item
		 * @param {float} coef.a slope of size_item function
		 * @param {float} coef.b size of the first items at the left or right of the selected item
		 * @param {float} coef.d minimum size of a unselected item
		 */ 
		function calc_coef(n,m,S,x0,Mx,coef) {
			var k_ = (m>n)? m : n;
			var d_ = (n<1)? ((m<1)? 0 : 1) : ((m<1)? 1 : 2);
			var smn_ = sum_n(n)+sum_n(m);
			var f_ = n+m+d_-smn_/k_;
			var Sw_ = S-Mx;

//			test to eliminate the case where S is so small that the selected item does not
//			fit in the container
			if (Mx>S){
				alert("please resize your window. A single items does not feet for the menu !!");
				exit();
			}

//			test to distinguish between :
//			- having enough space and start with the initial x0
//			- don't have enough space and x0 have to be modified 
			if (Sw_/x0 < f_)
			{
//				calculating new b(x0) using coef.d
				if (Sw_*k_/smn_>coef.d){
					coef.b = (Sw_-coef.d*smn_/k_)/f_;
					if(coef.d>coef.b){
						alert("please resize your window or change parameters. With configured parameter and this window size, menu items does not have enough room  !!");
						exit();
					}
					coef.a = (coef.d-coef.b)/k_;
				}
				else
				{
					alert("please re-size your window or change parameters. With configured parameter and this window size, menu items does not have enough room  !!");
					exit();
				}
			}
			else  
			{
//				calculating new a using x0
				coef.a = (Sw_-(n+m+d_)*x0)/smn_;
				coef.b = x0;
			}

		}
		/**
		 * Calculate the size of a item depending on the context. The coefficient of the linear equation have 
		 * been calculated by calc_coef with the needed context. However this function take into account the context
		 * only indirectly with the calculated coefficient. This function is just a linear equation with k an j to
		 * get the abscissa.
		 * @name $.fn.dmlhm#size_item
		 * @function
		 * @since version 0.1.0
		 * @inner
		 * @param {int} k index of the unselected item
		 * @param {int} j index of the selected item
		 * @param {obj} coef coefficient used to calculate the size of a item (linear equation)
		 * @param {float} coef.a slope of the line
		 * @param {float} coef.b constant of the line
		 * @param {float} coef.d minimum size of a unselected item
		 * @param {int} Mxw size of the selected item (k=j)
		 * @param {int} extra extra pixels of the box model to subtract to have the inner width
		 * @returns {float} inner width calculated by this function for item k with item j selected.
		 * 
		 */
		function size_item(k,j,coef,MxW,extra){
			var xi_;
			if (k<j) xi_ = Math.floor((coef.b+coef.a*(j-k-1))-extra);
			else if (k>j) xi_ = Math.floor((coef.b+coef.a*(k-j-1))-extra);
			else xi_ = MxW;
			if (xi_<1) { return(1);} // should never append. inner width of 1.
			else return(xi_);
		}

		/**
		 * Collection of jQuery object (DIV container) selected by the user and passed to the dmlhm method.
		 * @name $.fn.dmlhm#this
		 * @field
		 * memberOf $.fn.dmlhm
		 * @public
		 */
		/**
		 * Function defining the menu (dynamic DIV tree menu items) for each jQuery object (container)
		 * of the collection "this".
		 * @name each
		 * @function
		 * @memberOf $.fn.dmlhm.this
		 * @see $.fn.dmlhm#this
		 * @namespace Holds function which transform original HTML DOM in functional DIV tree menu.
		 */
		this.each(function() {
			var min_width = o.min_width; // the minimum width of an menu item
			var wmiu;                    // the fixed width of a unselected menu item 
			var wmis;                    // the fixed width of a selected menu item 
			var divpadd = 0;             // padding of an menu item : fixed by the first menu item css property (left)
			var divbord = 0;             // border of an menu item : fixed by the first menu item css property (left)
			var divmarg = 0;             // margin of an menu item : fixed by the first menu item css property (left)
			var divextra = 0;            // extra pixel that follow css box model to add or substract to have real width or container width
			var preCalcs = new Array();  // pre-calculate size of items depending on :
										 //     index of parent,
										 //      index of selected item
										 //      and index of item
			var nbm;                     // wall number of menu item
			var nbmpp = new Array();     // number of menu item per parent
			var idmpg = new Array();     // index from nbmpp of menu parent with nbm indexation
			var nbpm = 0;                // number of parent menu item
			var level = 0;               // level of a sub-menu
			var ind=-1;                  // index of the current parent item
			var indc;                    // index of the current selected item
			var pind = new Array();      // parent index table
			var jqol = new Array();      // Jquery object div list : one per developed menu or sub menu
			var jqdl;                    // jQuery object div list of container $(this)
			var strdiv;                  // string to select items of $(this) container
			var strlevel;                // string for class selection in $(this) container
			var container = $(this);     // container of menu item
			var li=container.find("li"); // list of menu item from ul/li menu

//			defining numbers of menu items, parent menu and sub-menu
			nbm = li.size()+1; // add one for root item
			nbpm = container.find("ul").size();
			nbmpp[0]=container.children("ul").children("li").size();

//			init string for selector
			strdiv = '#'+container.attr('id')+' div';
			strlevel = strdiv+'.level_';


//			Transform ul/li menu in tree div
			li.each(function(ind_) {
				var indp_ = li.index($(this).closest("ul").parent());
				var level_ = $(this).parents("ul").size()-1;
				var cli;
				if (indp_<0) indp_=0;
				else indp_++; // shift index for the root menu
//				if li is a parent (or have child) : a ul child is present 
				if ($(this).children("ul").size())
				{
					cli = $(this).clone();
					cli.children("ul").remove();
					$('<div class="level_'+level_+'_'+indp_+'" id="child" >'+cli.html()+'</div>').appendTo(container);
				}
//				otherwise it's a simple item
				else {
					$('<div class="level_'+level_+'_'+indp_+'" id="m'+ind_+'" >'+$(this).html()+'</div>').appendTo(container);

				}

			});

//			clean container by deleting old menu information(ul/li)
			container.children("ul").remove();

//			init div list
			jqdl = $(strdiv);

//			getting/setting sizes
			if (o.css>0){
				min_width=parseInt(jqdl.width()); // width perform on the first item of the list
			}
			divpadd=parseInt(jqdl.eq(0).css('padding-left').replace(/px/, ''));
			divbord=parseInt(jqdl.eq(0).css('border-left-width').replace(/px/, ''));
			divmarg=parseInt(jqdl.eq(0).css('margin-left').replace(/px/, ''));
			divextra=(divpadd+divbord+divmarg)*2;
			wmiu=divextra+min_width;
			wmis=divextra+o.max_width;

//			defining the structure
			jqol[0] = $(strlevel+'0_0');
			preCalcs[0] = new Array();
			jqol[0].each(function(j) {
				preCalcs[0][j] = new Array();
			});
			for(var i_ = 0; i_ < nbm; i_++) idmpg[i_]=0;
			jqdl.filter('#child').each(function(i) {
				i++; // add one for root parent
				preCalcs[i] = new Array();
				idmpg[jqdl.index($(this))+1]=i; // shift index for root menu
				var id_ = $(this).next().attr('class').split('_',3);
				var l_ = parseInt(id_[1]);
				var s_ = strlevel+id_[1]+'_'+id_[2];
				var jl_ = parseInt(id_[2]);
				var jql_ = $(s_);
//				take into account the parent or parents that stay when 
//				entering a sub menu : add to list of menu jql
				while (jl_ > 0)
				{
					var jqmp_ = jqdl.eq(jl_-1);
					var nid_ = jqmp_.attr('class').split('_',3);
					jql_=jql_.add(jqmp_); // As of Jquery 1.4 add use original dom order of object list. perfect for indexing !
					jl_ = parseInt(nid_[2]);
				}
				jqol[i]=jql_;
				nbmpp[i]=jql_.size();
				jql_.each(function(j) {
					preCalcs[i][j] = new Array();
				});
			});



//			binding menu size pres calculation to resize event
			$(window).bind('resize',
					/**
					 * Calculate "preCalcs" with the actual size of the container. "preCalcs" is the table containing all
					 * items size for all combination of  parent item (context)index, selected item index and item index.
					 * @name $.fn.dmlhm.this.each#resize
					 * @event
					 * @since version 0.1.0
					 * @public
					 * @param {obj} event event object (not used) 
					 */
					function(event){
				var maxmw_=parseInt(container.width());
				var coef_ = {
						a : 0,
						b : 0,
						d : o.min_width_abs+divextra
				};
				for (var i_ =0; i_< nbpm; i_++) {
					var toobig_ = (((nbmpp[i_]-1)*wmiu+wmis)>maxmw_);
					jqol[i_].each(function(j) {
						if (toobig_){
							var n = nbmpp[i_]-j-1;
							calc_coef(n,j,maxmw_,wmiu,wmis,coef_);
							jqol[i_].each(function(k) {
								preCalcs[i_][j][k] = size_item(k,j,coef_,o.max_width,divextra);
							});
						}
						else
						{
							jqol[i_].each(function(k) {
								if (k!=j) preCalcs[i_][j][k]=min_width;
								else preCalcs[i_][j][k]=o.max_width;
							});
						}
					});
				}
				anim_menu(idmpg[ind],jqdl.eq(indc),o.speed_menu);
			});

//			binding init function to item menu special event
			jqdl.bind(o.dmlhm_event_init,
					/**
					 * Initialize the context and set the target item as the selected one.
					 * <p>
					 *   Note : it doesn't do the same as the click_func as it is fired by
					 *   a custom event so the href is not in action except if this event
					 *   fired a click event on the link. Other difference come
					 *   from the lack of history here : The target may be deep in sub menu this
					 *   function will reconstruct all history. With click_func it's impossible. It 
					 *   have to follow the branch of the tree up to the target.
					 *   </p>
					 * @name $.fn.dmlhm.this.each#custom
					 * @event
					 * @since version 0.1.3
					 * @param {obj} event event object (not used)
					 */
					function (event) {
				var target_=$(this);
				var id_ = target_.attr('id');
				var level_ind_;
				var context_;
				// retrieve target context
				if(o.do_child_when_init>0 )
				{
					if (id_ == 'child')
					{
						level_ind_ = target_.next().attr('class').split('_',3);
					}
					else if (id_ == 'parent')
					{
						level_ind_ = target_.next().attr('class').split('_',3);
					}
					else
					{
						level_ind_ = target_.attr('class').split('_',3);
					}
				}
				else
				{
					level_ind_ = target_.attr('class').split('_',3);
				}

				// if this function is fired in a already initialized context
				if (ind>-1)
				{
					var nindc_ = jqdl.index(target_);
					context_ = idmpg[ind];
					unset_bind(jqol[context_]);
					// if it's aleready a selected item there is nothing to do
					// if not re-init all the context and visibles items
					if ( nindc_ != indc )
					{
						var nind_;
						var nlevel_;
						// when the selected item is the next item of a parent which is the
						// target, level_ind_ don't give the right information. But as it is
						// the only case where it's possible (target=selected is treated by 
						// the test above  nindc_ != indc and child.next=selected is not possible
						// as the next item of a chid item is always hidden) we can conclude :
						//     selected => same context/level
						if (level_ind_[0]=='selected')
						{
							nind_=ind;
							nlevel_=level;
						}
						else
						{
							nind_=parseInt(level_ind_[2]);
							nlevel_=parseInt(level_ind_[1]);
						}
						// if the target is not in the same context :
						// change context and selected item 
						if ( nind_ != ind )
						{
							// hide actuel (sub)menu
							jqol[context_].filter('#parent')
							.removeClass('parents')
							.attr('id','child');
							jqol[context_].filter('.selected').attr('class','level_'+level+'_'+ind);
							jqol[context_].hide();
							init_menu_with_history(target_,nind_,nlevel_,nindc_);
							context_ = idmpg[ind];
						}
						// if the target is in the same context :
						// change selected item 
						else
						{
							indc=nindc_;
							jqol[context_].filter('.selected').attr('class','level_'+level+'_'+ind);
							if (id_!='parent')
							{
								target_.attr('class','selected');
							}
						}
						// animation (showing) 
						anim_menu(context_,jqdl.eq(indc),o.speed_menu);
					}
				}
				else
				{
					init_menu_with_history(target_,parseInt(level_ind_[2]),parseInt(level_ind_[1]),jqdl.index(target_));
					context_ = idmpg[ind];
					// fire event resise to get initial sizes calculated ,
					// and submenu animation (showing)
					// note : this will recalculate all dmlhm size defined so far
					$(window).trigger('resize');
				}

				// perform action on the target if action on init is allowed
				if(o.do_action_when_init>0 )
				{
					// performe action on the target if "free"
					if (id_!= 'child' && id_ != 'parent')
					{
						var data ={
								id: id_,
								context:context_,
								target:target_,
								ind:ind,
								level:level,
								indc:indc,
								jqdl:jqdl,
								jqol:jqol,
								idmpg:idmpg
						};
						o.click_func_action(data);
					}
				}
				if(o.do_href_when_init>0 )
				{
					// only do the link action by firer clik event
					target_.child('a').trigger('click');
				}

				// bind the (sub) menu
				set_bind(jqol[context_]);
			});

			// initialize the context and initial sizes with chosen item selected
			jqdl.eq(o.starting_item).trigger(o.dmlhm_event_init);

			//end of treatment
			//function after that

			/**
			 * animate menu items depending on action (mouse movement or click) and
			 * context (witch item is selected, the (sub)menu activated, ...)
			 * @name $.fn.dmlhm.this.each#anim_menu
			 * @function
			 * @inner
			 * @since version 0.1.0
			 * @param {int} context index of active sub-menu 
			 * @param {obj} target jQuery object who becomes "selected"
			 * @param {int} speed duration in milliseconds for the animation
			 */
			function anim_menu(context,target,speed)
			{
				var j_ = jqol[context].index(target);
				jqol[context].each(function(k) {
					$(this).animate({ width : preCalcs[context][j_][k] }, { queue:false , duration:speed});
				});
			}
			/**
			 * It initialize menu items depending on context
			 * and reconstruct history (pind, ...) if needed.
			 * @name $.fn.dmlhm.this.each#init_menu_with_history
			 * @function
			 * @since version 0.1.5
			 * @inner
			 * @param {obj} target jQuery object who becomes "selected"
			 * @param {int} nind new index of parent
			 * @param {int} nlevel new level for target (sub)menu
			 * @param {int} nindc new index for "selected"
			 *
			 */
			function init_menu_with_history(target,nind,nlevel,nindc)
			{
				var l_=0;
				var i_=0;
				var context_;
				// initialize contexte
				ind=nind;
				indc=nindc;
				level=nlevel;
				context_=idmpg[ind];
				// reconstrute history if in sub menu
				if (ind > 0)
				{
					jqol[context_].each(function(k) {
						if ($(this).attr('id') == 'child')
						{
							var curlevel_ = $(this).attr('class').split('_',3);
							// check that child items are not at the same level as the target
							if ( level != parseInt(curlevel_[1]))
							{
								pind[l_]=i_;
								$(this).attr('id','parent').addClass('parents');
								l_++;
								i_=jqdl.index($(this))+1; // shift index for root
							}
						}
					});
				}
				// check consistency
				if ( (level != l_) || (ind != i_) )
				{
					alert('Oops,sorry Bug! Bug!');
					exit();
				}
				// change target's class
				if (target.attr('id')!='parent')
				{
					target.attr('class','selected');
				}
			}
			/**
			 * For menu items given (target), bind click,mousse-enter,mouse-leave event to special function.
			 * @name $.fn.dmlhm.this.each#set_bind
			 * @function
			 * @since version 0.1.0
			 * @inner
			 * @param {obj} target jQuery object list to bind
			 */
			function set_bind(target) {
				target.bind("click",  click_func);
				target.bind("mouseenter",  mouseenter_func);
				target.bind("mouseleave",  mouseleave_func);
			};

			/**
			 * For menu items given (target), unbind click,mousse-enter,mouse-leave event from special function.
			 * @name $.fn.dmlhm.this.each#unset_bind
			 * @function
			 * @since version 0.1.0
			 * @inner
			 * @param {obj} target jQuery object list to unbind
			 */
			function unset_bind(target) {
				target.unbind("mouseenter",  mouseenter_func);
				target.unbind("mouseleave",  mouseleave_func);
				target.unbind("click",  click_func);
			};

			/**
			 * When mouse leave a item which is not the selected one go
			 *  back to it. Do nothing otherwise.
			 * @name $.fn.dmlhm.this.each#mouseleave_func
			 * @event
			 * @since version 0.1.0
			 * @param {obj} event Event object (not used).  
			 */
			function mouseleave_func(event) {
				if (jqdl.index($(this)) != indc)
				{
					var sel_ = jqdl.eq(indc);
					anim_menu(idmpg[ind],sel_,o.speed_menu);
					if (sel_.attr('id') != 'parent') sel_.attr('class','selected');
					if ($(this).attr('id') != 'parent') $(this).attr('class','level_'+level+'_'+ind);
				}
			};
			/**
			 * When mouse enter a item, change it in selected item if not parent.
			 * Adapt size in all case. It is a temporary operation as mouse-leave set 
			 * back the item to the initial state.  
			 * @name $.fn.dmlhm.this.each#mouseenter_func
			 * @event
			 * @since version 0.1.0
			 * @param {obj} event Event object (not used).
			 */
			function mouseenter_func(event) {
				anim_menu(idmpg[ind],$(this),o.speed_menu);
				$(this).siblings('.selected').attr('class','level_'+level+'_'+ind);
				if ($(this).attr('id') != 'parent')
				{
					$(this).attr('class','selected');
				}
			};
			/**
			 * When click, define permanently item as selected and perform
			 * actions :<ul>
			 *    <li> develop sub-menu and hide current siblings item</li>
			 *    <li> close sub-menu(s) and show parent(s) (sub-)menu.</li>
			 *    <li> perform href action of the link (return true)</li>
			 *    <li> perform eventually a additional action</li>
			 *</ul>
			 * @name $.fn.dmlhm.this.each#click_func
			 * @event
			 * @since version 0.1.0
			 * @param {obj} event Event object (not used).
			 */
			function click_func(event){
				var id_ = $(this).attr('id');
				var context_=idmpg[ind];
				var target_=$(this);
				indc=jqdl.index(target_);
				if(id_=='child') {
					event.stopImmediatePropagation();
					unset_bind(jqol[context_]);
					pind[level]=ind;
					$(strlevel+level+'_'+ind).hide(o.speed_menu_hide);
					target_.attr('class','level_'+level+'_'+ind).addClass('parents');
					level++;
					ind=indc+1; // shift index for root
					context_=idmpg[ind];
					anim_menu(context_,target_,o.speed_menu_show);
					target_.attr('id','parent');
					set_bind(jqol[context_]);
				}
				else if(id_=='parent') {
					event.stopImmediatePropagation();
					unset_bind(jqol[context_]);
					target_.removeClass('parents');
					target_.nextAll('#parent')
					.attr('id','child')
					.removeClass('parents');
					var clevel=target_.attr('class').split('_',2);
					target_.siblings('.selected').attr('class','level_'+level+'_'+ind);
					for (var i = level;i>clevel[1];i--)
					{
						$(strlevel+level+'_'+ind).hide(o.speed_menu_hide);
						level--;
						ind=pind[level];
					}
					context_=idmpg[ind];
					indc=jqdl.index(target_);
					target_.attr('class','selected').attr('id','child');
					anim_menu(context_,target_,o.speed_menu_show);
					set_bind(jqol[context_]);
				}
				else
				{
					var data ={
							id:id_,
							context:context_,
							target:target_,
							ind:ind,
							level:level,
							indc:indc,
							jqdl:jqdl,
							jqol:jqol,
							idmpg:idmpg
					};
					o.click_func_action(data);
				}
				return true;
			};

		});

		return this;

	};

	/**
	 * Defaults options : user may change this object instead of providing options as argument of the dmlhm method. 
	 * See <a href="../../demo/howto.html">the howto page</a> for a descriptions of these options. 
	 * @name defaults
	 * @field
	 * @since version 0.1.0
	 * @memberOf $.fn.dmlhm
	 * @public
	 * @property {int} speed_menu speed of the animation (mainly mouse hover/out on menu items) in milliseconds
	 */
	$.fn.dmlhm.defaults = {
			speed_menu          : 900,                 // speed of the animation (mainly mouse hover/out on menu items) in milliseconds
			speed_menu_show     : 2000,                // speed in milliseconds for menu level to show
			speed_menu_hide     : 1500,                // speed in  milliseconds for menu level to hide (expected to be faster then show => less then speed_menu_show)
			max_width           : 250,                 // the maximum width of an menu item
			min_width           : 150,                 // the minimum width of an menu item
			min_width_abs       : 5,                   // the absolute minimum width of an menu item
			css                 : 1,                   // use css to fix min_width (0 no, 1 yes default)
			starting_item       : 0,                   // default starting item to be selected (index of li chosen) 
			dmlhm_event_init    : 'init_event',        // default name for dmlhm custom event bind to init function 
			do_action_when_init : 0,                   // if 1 action function of click event is use with target (if free) of initialisation event 
			do_href_when_init   : 0,                   // if 1 href of link's target is take into account when a initialization event is fired
			do_child_when_init  : 1,                   // if 1 when target of initialization event is a child item it is transformed in a parent one 
			click_func_action   : function (){}        // default action function for click event (for now, only used by free item (nor child or parent item)
	};

})(jQuery);
